--[[--

   parameters

--]]

local params = {}

local coupi   = require 'coupi'
local shape   = require 'coupi.shape'
local control = require 'coupi.control'

local vec3d   = require 'coupi.vec3d'
local rot3d   = require 'coupi.rot3d'
local quat    = require 'coupi.quat'

params.box = {
   x = 12,
   y = 18,
   z = 8,
}

params.g = -9.8                 -- m/s^2, free fall accel.

-------------------------------------------------------------------
--                          Water level and markers
-------------------------------------------------------------------
params.water = {}

params.water.level = 0.5 * params.box.z
params.water.r = 0.01           -- size of markers, m
params.water.dr = 0.1           -- approx. distance between markers, m

-------------------------------------------------------------------
--                           Debris parameters
-------------------------------------------------------------------

params.tree = {}
params.tree.boundingR = 1.2     -- m
params.tree.Cd = 0.1            -- Reynold drag coefficient: it is
                                -- typically around 1, however, we can
                                -- also replace it by a function of
                                -- velocity stream in the future

-------------------------------------------------------------------
--                           Cylinder parameters
-------------------------------------------------------------------

params.cylinder    = {}
params.cylinder.r  = 0.05 -- radius of spheres it is built from, m
params.cylinder.dr = params.cylinder.r -- distance between elements
params.cylinder.m  = 476.0       -- mass, kg
params.cylinder.D  = 1.12        -- cylinder diameter, m
params.cylinder.R  = 0.5 * params.cylinder.D -- radius of cylinder
params.cylinder.H  = 0.56                    -- height
params.cylinder.depth = 0.41                 -- m, how deep under water

params.cylinder.Cd = 1.0        -- Reynold drag coefficient: it is
                                -- typically around 1, however, we can
                                -- also replace it by a function of
                                -- velocity stream in the future

-- crossection under water m^2:
params.cylinder.Ax = params.cylinder.depth * params.cylinder.D

-- moment of inertia is used for cylindrical shell with open ends
params.cylinder.Iz = 
   params.cylinder.m * params.cylinder.r * params.cylinder.r

-- special friction for omega attenuation
params.cylinder.cw = 1.0

params.cylinder.y  = 0.7 * params.box.y

-- calculating the position of center of mass
params.cylinder.z = params.water.level - 
   params.cylinder.depth + 0.5 * params.cylinder.H

-- Cylinder position
params.cylinder.c = { params.box.x/2, 
                      params.cylinder.y,
                      params.cylinder.z }

-- Cylinder dynamic model

-- modes:
params.cylinder.modes = {
   ROTATION_ONLY = 0,           -- can only rotate around z-axis
   SPRING1D = 1,                -- can rotate + linear spring/damper
                                -- along y
}

params.cylinder.mode = params.cylinder.modes[ 'SPRING1D' ]

params.cylinder.spring = {}
params.cylinder.spring.k = 1000 -- N/m, stiffness of the spring, default 1000 N/m, tried 1 to 100000
params.cylinder.spring.c = 1  -- Ns/m, damping coefficient, default 100, tried 1 to 
params.cylinder.spring.m = params.cylinder.m     -- mass of the system
params.cylinder.spring.cy = params.cylinder.c[2] -- zero point

-------------------------------------------------------------------
--                           Hull (pontons) parameters
-------------------------------------------------------------------

params.hull = {}
params.hull.r   = params.cylinder.r
params.hull.dr  = params.cylinder.dr
params.hull.m   = 544.0         -- kg, each!
params.hull.L   = 6.1           -- m
params.hull.H   = 0.61          -- m
params.hull.angle = 30          -- degrees between hulls

-- degrees in rads
params.hull.angle_rad = math.pi * params.hull.angle / 180.0

params.hull.depth = params.cylinder.depth -- under water

-- shifts of edges relative to the cylinder
params.hull.x0 = 0.75 * params.cylinder.R
params.hull.y0 = 0.75  * params.cylinder.R

-- calculating the position of center of mass
local xdist = params.hull.x0 + 
   0.5 * params.hull.L * math.sin( params.hull.angle_rad/2 )

params.hull.x1 = params.cylinder.c[1] + xdist
params.hull.x2 = params.cylinder.c[1] - xdist

params.hull.y = params.cylinder.c[2] - params.hull.y0 - 
   0.5 * params.hull.L * math.cos( params.hull.angle_rad/2 )
params.hull.z = params.water.level - 
   params.hull.depth + 0.5 * params.hull.H

-- Cylinder position
params.hull.c1 = { params.hull.x1,
                   params.hull.y,
                   params.hull.z }

params.hull.c2 = { params.hull.x2,
                   params.hull.y,
                   params.hull.z }

-------------------------------------------------------------------
--                           Materials
-------------------------------------------------------------------

-- Materials (rho - density in kg/m^3, G - stiffness in GPa, nu -
-- Poisson coefficent, mu - Coloumb friction (static/dyn), CR -
-- restitution coefficient.

params.material = {}

params.material.wetwood = { name = 'wetwood',
                            rho = 650.0, 
                            G   = 0.0001, 
                            nu  = 0.27, 
                            mu  = 0.3, 
                            CR  = 0.1 }
-- CR was 0.55

params.material.cylinder = { name = "cylinder",
                             G   = 0.0001, 
                             nu  = 0.27,
                             mu  = 0.5,
                             CR  = 0.1,
                             rho = 4000,
                             gamma = 0, }
-- CR was 0.6

params.material.hull = { name = "hull",
                         G   = 10,
                         nu  = 0.27,
                         mu  = 0.5,
                         CR  = 0.1,
                         rho = 4000,
                         gamma = 0, }
-- CR was 0.6

params.material.water = { name = 'water',
                          rho  = 1000.0,
                          G    = 0.1,
                          nu   = 0.25,
                          mu   = 0.1,
                          CR   = 0.1, }


-------------------------------------------------------------------
--                         Water currents
-------------------------------------------------------------------

local l_ = params.water.level
local v0_ = - 2.0               -- m/s
local a_ = v0_ / ( 0.85 * 0.85 * l_ - 1.7 * 0.85 * l_ )
local b_ = -1.7 * a_ * l_

-- input: vec3d: position
function params.current_v( c )
   --   local v = vec3d:new( 0, a_ * c.v[3] * c.v[3] + b_ * c.v[3], 0 )
   local v = vec3d:new( 0, v0_, 0 )
   return v
end

-------------------------------------------------------------------
-- Some derivative values and functions
-------------------------------------------------------------------

-------------------------------------------------------------------
--   Restoring rotation quaternions for the hulls (pontons)
-------------------------------------------------------------------

local angle = math.pi * params.hull.angle / 180.0

local e1 = quat:new( {0,0,1}, math.pi/2 )
local e2 = quat:new( {0,1,0}, math.pi/2 )

local qplus = quat:new( { 0,0,1}, angle / 2 )
local qminus = quat:new( { 0,0,1}, - angle / 2 )

params.hull.q1 = qplus * e2 * e1
params.hull.q2 = qminus * e2 * e1

-------------------------------------------------------------------

return params
