--[[--

   Main rddp test
   
   Author: Anton Kulchitsky

   Usage: coupi rddp.lua

--]]

local coupi    = require 'coupi'
local control  = require 'coupi.control'
local mathx    = require 'coupi.mathx'
local aux      = require 'coupi.aux'
local sphtool  = require 'coupi.sphtool'

local vec3d    = require 'coupi.vec3d'
local rot3d    = require 'coupi.rot3d'
local quat     = require 'coupi.quat'

local libbasic = require 'libbasic'
local libdebris = require 'libdebris'
local libfloat  = require 'libfloat'

local params    = require 'parameters'


-- Time/saving parameters
coupi.time = {
   start = 0.0,                 -- starting time
   stop  = 10.0,                -- ending time
   dt    = 0.00001,           -- 10000 ns timestep
   save  = 0.1                 -- saving timestep
}

-- The area of simulation and gravity
coupi.box = params.box
coupi.g   = params.g

coupi.bsphere = {}
coupi.bsphere.drmin = 0.1

----------------------------------------------------------------
--- initialize the random number generator
----------------------------------------------------------------
math.randomseed( os.time() )

-------------------------------------------------------------------
--                           materials
-------------------------------------------------------------------

coupi.material:add( params.material.wetwood )
coupi.material:add( params.material.cylinder )
coupi.material:add( params.material.hull )
coupi.material:add( params.material.water )

local friction_cylinder_debris =
   coupi.material:select_cmprop( 'cylinder', 'wetwood' )
friction_cylinder_debris = params.mu_cylinder_debris

local friction_hull_debris =
   coupi.material:select_cmprop( 'hull', 'wetwood' )
friction_hull_debris = params.mu_cylinder_debris

coupi.boundary.material = 'wetwood'
coupi.boundary.opentop = true   -- no cover

-------------------------------------------------------------------
--                           groups
-------------------------------------------------------------------

-- Groups. We have to add all group names we are going to use, water
-- does not interact with the rest (as a rigid body)
coupi.atom.group:add( 'debris', 
                      'cylinder', 
                      'cylinder-stripe', 
                      'hull', 
                      'water' )

coupi.atom.group:avert( 'debris', 'water' )
coupi.atom.group:avert( 'cylinder', 'water' )
coupi.atom.group:avert( 'cylinder-stripe', 'water' )
coupi.atom.group:avert( 'hull', 'water' )

-- cylinder will not interact with hulls either
coupi.atom.group:avert( 'hull', 'cylinder' )

-------------------------------------------------------------------
--                           cylinder
-------------------------------------------------------------------

local cyl = sphtool.largecylinder( params.cylinder.H,
                                   params.cylinder.R,
                                   params.cylinder.r,
                                   params.cylinder.dr,
                                   'cylinder', 'cylinder' )

cyl.body.c = params.cylinder.c
cyl.body.q = {1,0,0,0}
cyl.body.m = params.cylinder.m
cyl.body.w = { 0, 0, 0 } -- initial rotation

cyl.body.control = 1
cyl.body.extforce = nil
cyl.body.monitor = 1

print( 'Number of atoms in cylinder: ', #cyl.atoms )

local ig_stripe = coupi.atom.group:resolve( 'cylinder-stripe' )

-- Adding stripes to the cylinder by changing a group for some atoms
for i,A in pairs(cyl.atoms) do
   if ( A.sbb_c[1]*A.sbb_c[2] < 0 ) then
      A.group = ig_stripe
   end
end

coupi.append( cyl )

-------------------------------------------------------------------
--                           hulls (pontons)
-------------------------------------------------------------------

local hull                      -- for both hulls

hull = sphtool.rectangle( params.hull.L,
                          params.hull.H,
                          params.hull.r,
                          params.hull.dr,
                          'hull', 'hull' )

hull.body.c = params.hull.c1
hull.body.q = params.hull.q1.q
hull.body.m = params.hull.m

hull.body.control = control.builtin.still
hull.body.monitor = nil

print( 'Number of atoms in a hull (each ponton):', #hull.atoms )

coupi.append( hull )

-------------------------------------------

hull = sphtool.rectangle( params.hull.L,
                          params.hull.H,
                          params.hull.r,
                          params.hull.dr,
                          'hull', 'hull' )

hull.body.c = params.hull.c2
hull.body.q = params.hull.q2.q
hull.body.m = params.hull.m

hull.body.control = control.builtin.still
hull.body.monitor = nil

print( 'Number of atoms in a hull (each ponton):', #hull.atoms )

coupi.append( hull )

-------------------------------------------------------------------
--                    object
-------------------------------------------------------------------

local tree = libdebris.tshape05( params.tree.boundingR, 
                                 'debris', 
                                 'wetwood' )

tree.body.c = { 0.50 * coupi.box.x,
                0.90 * coupi.box.y,
                params.water.level }

tree.body.v = { 0, 0, 0 }

tree.body.monitor     = 1
tree.body.control     = nil
tree.body.extforce    = 1
tree.body.positioning = 1

-- Setting it to not interact with x/y boundaries
tree.body.noxbound = true
tree.body.noybound = true

-- body orientation
tree.body.q = {0.3981928315005232,0.5283245696648566,0.3284714780980173,0.6741083786055059}

-- append debris
coupi.append( tree )

-------------------------------------------------------------------
--                      Water markers
-------------------------------------------------------------------

local wps = libfloat.watermarkers( params.water.level,
                                   params.water.r,
                                   params.water.dr,
                                   params.box,
                                   'water',
                                   'water' )
for i, wp in pairs(wps) do
   coupi.append( wp )
end

-------------------------------------------------------------------
--                      Monitor
-------------------------------------------------------------------

local cyl_data = {
   params.cylinder.c[1],
   params.cylinder.spring.cy,
   params.cylinder.c[3],
}             -- to store cylinder parameters

local function monitor( i, m, V,
                        Ix, Iy, Iz,
                        cx, cy, cz,
                        vx, vy, vz,
                        wx, wy, wz,
                        fx, fy, fz,
                        tx, ty, tz )

   -- just update tree_data to use in aftertimestep and control functions
   cyl_data.c = { cx, cy, cz }
   cyl_data.w = { wx, wy, wz }
   cyl_data.f = { fx, fy, fz }
   cyl_data.t = { tx, ty, tz }

   -- Adding the water resistance force to the cylinder (averaged on
   -- center)

   local C = vec3d:new( cx, cy, cz ) -- center
   local wv = params.current_v( C ) -- current water velocity at center
   local wvabs = wv:abs()           -- |v_w|
   local Cd = params.cylinder.Cd
   local rho = params.material.water.rho
   local A = params.cylinder.Ax

   local K = 0.5 * Cd * rho * wvabs * A

   local wF = wv:scale( K )     -- resistance force

   cyl_data.wf = wF.v           -- reference to the data
end

coupi.monitor_functions = {
   monitor,
}

-------------------------------------------------------------------
--                      Control
-------------------------------------------------------------------

-- rotating cylinder
local function cylcontrol ( i, dt, m, 
                            vx, vy, vz, wx, wy, wz, 
                            fx, fy, fz, tx, ty, tz )

   local cw = params.cylinder.cw
   local Iz = params.cylinder.Iz

   -- spring case
   local vynew = 0
   if params.cylinder.mode == params.cylinder.modes[ 'SPRING1D' ] then
      local M = params.cylinder.spring.m
      local Cu = params.cylinder.spring.c * dt / M
      local freq2 = params.cylinder.spring.k * dt / M
      local dc = params.cylinder.spring.cy - cyl_data[2]
      vynew = vy + freq2 * dc - Cu * vy + fy * dt / M
   end
   
   local wznew = wz + dt * ( tz  - cw * wz ) / Iz
      
   return 0, vynew, 0, 0, 0, wznew
end

coupi.control_functions = {
   cylcontrol,
}

-------------------------------------------------------------------
--                      External Force
-------------------------------------------------------------------

local extforce = libfloat.bouy_and_drag_gen( 
   tree,
   params.material.water.rho,
   params.material.wetwood.rho,
   params.tree.Cd,
   params.water.level,
   params.current_v )

coupi.extforce_functions = {
   extforce,
}

-------------------------------------------------------------------
--                      Repositioning
-------------------------------------------------------------------

-- margins for periodic function to avoid segfaults when bodies move
-- outside
local DSX = params.tree.boundingR
local DSY = DSX

local repos = libfloat.repos_gen( DSX, params.box.x - DSX,
                                  DSY, params.box.y - DSY )

coupi.position_functions = {
   repos,
}

-------------------------------------------------------------------
--                      After time step: save
-------------------------------------------------------------------

-- We save position and forces 
local ff = assert( io.open( 'cylinder.txt', 'w' ) )

local s1 = string.format( '# Cylinder data\n#\n' )
local s2 = string.format( '# Angular velocity, forces and torques:\n'..
                             '# f - all force without water resistance\n' ..
                             '# fw - water resistance force\n' )
local s3 =  string.format( '# [1]t ' .. 
                              '[2]wz,rad/s ' ..
                              '[3]fx,N [4]fy,N [5]fz,N ' ..
                              '[6]fwx,N [7]fwy,N [8]fwz,N ' ..
                              '[9]tx,Nm [10]ty,Nm [11]tz,Nm\n' )

ff:write( s1 )
ff:write( s2 )
ff:write( s3 )

local function aftertimestep( t, dt, n )
   -- Saving to the file each 100th step
   if n % 100 == 0 then
      local output =
         string.format(
            '%2.10f,  %f,  %f, %f, %f,  %f, %f, %f, %f, %f, %f\n',
            t, cyl_data.w[3],
            cyl_data.f[1], cyl_data.f[2], cyl_data.f[3],
            cyl_data.wf[1], cyl_data.wf[2], cyl_data.wf[3],
            cyl_data.t[1], cyl_data.t[2], cyl_data.t[3] )
      ff:write( output )
      ff:flush()
   end

end

coupi.callback.aftertimestep = aftertimestep

-------------------------------------------------------------------
--                    Finalize callback functions
-------------------------------------------------------------------

local function finalize( t )
   print( string.format( 
             '*** FINILIZE FUNCTION CALLED at t = %3.3f ***\n', t ) )
   return 'rddp-final.h5'
end

coupi.callback.finalize = finalize


-------------------------------------------------------------------
--                      Display options
-------------------------------------------------------------------
coupi.callback.presave   = coupi.callback.std.presave( 'rddp-' )
coupi.callback.aftersave = coupi.callback.std.aftersave_long()

coupi.contact_hash = {}
coupi.contact_hash.capacity = 100000


